// -*- C++ -*-
//
// eventChecker.cc
// 
// Mon Jul  8 22:27:07 JST 2002
// M.Yamaga
//
// $Id: eventChecker.cc,v 1.3 2002/10/11 17:01:47 yamaga Exp $
//
// $Log: eventChecker.cc,v $
// Revision 1.3  2002/10/11 17:01:47  yamaga
// updates.
//
// Revision 1.2  2002/10/11 16:39:17  yamaga
// updated for new eventChecker code.
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <list>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include "midas.h"
#include "E391Detector/E391Detector.h"
#include "MidasManager/MidasManager.h"


// globals
int  Debug = 0;
int  A0evnumSlot = 18;
int  A1evnumSlot = 18;
int  T0evnumSlot = 16;
int  T1evnumSlot = 17;
int  A0evnumChannel = 88;
int  A1evnumChannel = 88;
int  T0evnumChannel = 56;
int  T1evnumChannel = 56;

E391Detector&  e391det = E391Detector::instance();

/* ODB structures */
RUNINFO  runinfo;

//////// user code is defined in the user_ana.cc
extern void  user_init();
extern void  user_begin_of_run( int runnum );
extern void  user_process_event( int runnum,int spillnum,int eventnum );
extern void  user_process_event_exp( HNDLE hbuf,HNDLE request_id,
                                     EVENT_HEADER* pheader, void* pevent );
extern void  user_end_of_run( int runnum );
extern void  user_term();


////////
// usage
static void
usage( int argc, char* argv[] )
{
  std::cout << "usage : " << argv[0] << " [options]"
            << std::endl 
            << "  options:" << std::endl
            << "    -h <hostname> (connect to the running experiment.)"
            << std::endl 
            << "    -i <filenames...> (you can use %05d for run_number."
            << " e.g. run%05d.mid)"
            << std::endl 
            << "    -r <run#from> <run#to> (work with %05d)"
            << std::endl
            << "    -n <#event>"
            << std::endl 
            << "    -skip <skipevent>"
            << std::endl
            << "    -en_adc0 <slot> <ch>" << std::endl
            << "    -en_adc1 <slot> <ch>" << std::endl
            << "    -en_tdc0 <slot> <ch>" << std::endl
            << "    -en_tdc1 <slot> <ch> (event_number input channel)" 
            << std::endl;
}


////////
// main
int
main( int argc, char* argv[] )
{
  if ( argc < 2 ){
    usage( argc,argv );
    return( 1 );
  }

  // command parser
  int  Skipevent = 0;
  int  Nevent  = -1;
  int  runFrom = 0;
  int  runTo   = 0;
  std::list<std::string>  iflist;
  std::list<std::string>  iftmp;
  char  exphost[64];
  for( int i=1;i<argc;i++ ){
    if ( argv[i][0] == '-' ){
      if ( strcmp( argv[i],"-h" ) == 0 ){
        if ( ++i<argc && argv[i][0] != '-' ){
          strcpy( exphost,argv[i] );
        }
        else
          i--;
      }
      else if ( strcmp( argv[i],"-i" ) == 0 ){
        while( ++i<argc && argv[i][0] != '-' ){
          iftmp.push_back( argv[i] );
        }
        i--;
      }
      else if ( strcmp( argv[i],"-r" ) == 0 ){
        runFrom =  strtol( argv[++i],NULL,0 );
        runTo   =  strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-n" ) == 0 ){
        Nevent = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-skip" ) == 0 ){
        Skipevent = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-en_adc0" ) == 0 ){
        if ( ++i < argc && argv[i][0] != '-' ){
          A0evnumSlot = strtol( argv[i],NULL,0 );
          if ( ++i < argc && argv[i][0] != '-' )
            A0evnumChannel = strtol( argv[i],NULL,0 );
          else
            i--;
        }
        else
          i--;
      }
      else if ( strcmp( argv[i],"-en_adc1" ) == 0 ){
        if ( ++i < argc && argv[i][0] != '-' ){
          A1evnumSlot = strtol( argv[i],NULL,0 );
          if ( ++i < argc && argv[i][0] != '-' )
            A1evnumChannel = strtol( argv[i],NULL,0 );
          else
            i--;
        }
        else
          i--;
      }
      else if ( strcmp( argv[i],"-en_tdc0" ) == 0 ){
        if ( ++i < argc && argv[i][0] != '-' ){
          T0evnumSlot = strtol( argv[i],NULL,0 );
          if ( ++i < argc && argv[i][0] != '-' )
            T0evnumChannel = strtol( argv[i],NULL,0 );
          else
            i--;
        }
        else
          i--;
      }
      else if ( strcmp( argv[i],"-en_tdc1" ) == 0 ){
        if ( ++i < argc && argv[i][0] != '-' ){
          T1evnumSlot = strtol( argv[i],NULL,0 );
          if ( ++i < argc && argv[i][0] != '-' )
            T1evnumChannel = strtol( argv[i],NULL,0 );
          else
            i--;
        }
        else
          i--;
      }
      else if ( strcmp( argv[i],"-d" ) == 0 ){
        Debug = 1;
      }
      else{
        std::cout << " unknown option : " << argv[i] << std::endl;
        usage( argc,argv );
        return( 1 );
      }
    }
    else{
      std::cout << " unknown option : " << argv[i] << std::endl;
      usage( argc,argv );
      return( 1 );
    }
  }
    
  // iflist
  for( std::list<std::string>::iterator i = iftmp.begin();
       i != iftmp.end(); i++ ){
    if ( strchr( i->c_str(),'%' ) ){
      for( int rn=runFrom; rn<=runTo; rn++ ){
        char  s[1024];
        sprintf( s,i->c_str(),rn );
        iflist.push_back( std::string( s ) );
      }
    }
    else{
      iflist.push_back( *i );
    }
  }
  
  // call user init routine
  user_init();
  
  // main loop
  if ( iflist.size() > 0 ){
    // read from file
    for( std::list<std::string>::iterator f = iflist.begin();
         f != iflist.end(); f++ ){
      // open data file
      MidasFileIO  mfile( *f,O_RDONLY|O_LARGEFILE );
      if ( mfile.good() ){
        std::cout << "Opening input file : " << *f << std::endl;
        
        // event bank
        EVENT_HEADER*  pheader = mfile.bank()->eventHeader();
        void*          pevent  = mfile.bank()->eventBuffer();
        
        // Begin_of_run record
        int  run_number = -1;
        if ( mfile.readBOR() ){
          run_number = pheader->serial_number;
          std::cout << "Run = " << run_number << std::endl;
          e391det.setRunNumber( run_number );
          // get DB for this run
          e391det.getDB();
          
          // call user begin_of_run routine
          user_begin_of_run( run_number );
        }
        
        // Process Event
        mfile.skipEvent( 0,Skipevent );
        for( int i=0; i<Nevent || Nevent<0; i++ ){
          int  id = mfile.readEvent(0);
          if ( (id & 0x8000) == 0x8000 ){
            // Special event (BOR,EOR,MES)
            if ( id == EVENTID_BOR ){
              run_number = pheader->serial_number;
              std::cout << "Run=" << run_number << std::endl;
              e391det.setRunNumber( run_number );
              // get DB for this run
              e391det.getDB();
              
              // call user begin_of_run routine
              user_begin_of_run( run_number );
            }
            else if ( pheader->event_id == EVENTID_EOR ){
              // call user end_of_run routine
              user_end_of_run( run_number );
            }
          }
          else if ( id > 0 ){
            e391det.setEvent( pheader, pevent );
            e391det.unpack( 1 ); // only check the first event.
            int  spill_number = e391det.spillNumber();
            int  event_number = e391det.eventNumber();
            // call user analysis routine
            user_process_event( run_number,spill_number,event_number );
          }
          else{
            // end of file
            break;
          }
        }
        
      }
      else{
        std::cout << "No such file : " << *f << std::endl;
      }
      
    }
    
  }
  ////
  else{
    // connect to experiment
    int  status = cm_connect_experiment( exphost,"",
                                         "EventChecker",NULL );
    if ( status != CM_SUCCESS )
      return( -1 );
    
    /* open ODB structures */
    RUNINFO_STR(runinfo_str);
    HNDLE hDB, hKey;
    char  str[80];
    cm_get_experiment_database(&hDB, NULL);
    db_create_record(hDB, 0, "/Runinfo", strcomb(runinfo_str));
    db_find_key(hDB, 0, "/Runinfo", &hKey);
    if ( db_open_record(hDB,hKey,&runinfo, sizeof(runinfo),MODE_READ,NULL,NULL)
         != DB_SUCCESS ){
      std::cout << "cannot open /Runinfo." << std::endl;
      return 0;
    }

    HNDLE  hbuf;
    int  request_id;
    bm_open_buffer( "SYSTEM",EVENT_BUFFER_SIZE,&hbuf );
    bm_request_event( hbuf, 1, TRIGGER_ALL, GET_SOME,
                      &request_id, user_process_event_exp );
    do{
      status = cm_yield( 1000 );
    } while( status != RPC_SHUTDOWN &&
             status != SS_ABORT );

    cm_disconnect_experiment();
  }

  // call user term routine
  user_term();

  //
  return( 0 );
}

