// -*- C++ -*-
//
// NGF.cc
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Sep 25 22:32:13 JST 2001
//
// $Id: Ngf.cc,v 1.2 2002/12/05 14:42:01 yamaga Exp $
//
// Revision History
//
// $Log: Ngf.cc,v $
// Revision 1.2  2002/12/05 14:42:01  yamaga
// Minor fix.
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <iomanip>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>
#include <sys/vme.h>
#include <sys/vui.h>
#include "ngf/Ngf.h"


//////
#define DEBUG  0


//////
static const int  ngfmapsize   = 0x00100000;
static const int  SFI_DMA_SIZE = 0x00040000;


//////
Ngf::Ngf()
  : m_debugLevel( 0 ),
    m_baseaddr( 0 ),
    m_fd32( -1 ),
    m_map( (unsigned int*)-1 )
{
#if DEBUG
  std::cout << "Ngf::Ngf()" << std::endl;
#endif
};


//////
Ngf::Ngf( unsigned int baseaddr, int debugLevel )
  : m_debugLevel( debugLevel ),
    m_baseaddr( baseaddr ),
    m_fd32( -1 ),
    m_map( (unsigned int*)-1 )
{
#if DEBUG
  std::cout << "Ngf::Ngf(addr)" << std::endl;
#endif

  // initialize
  if ( !init( m_baseaddr,m_debugLevel ) ){
    std::cout << "Ngf::Ngf(addr) : init failed" << std::endl;
    exit( 1 );
  }

};


////// 
Ngf::Ngf( const Ngf& n )
  : m_debugLevel( 0 ),
    m_baseaddr( 0 ),
    m_fd32( -1 ),
    m_map( (unsigned int*)-1 )
{
  std::cerr << "Ngf::Ngf(copy) : You cannot copy an NGF object."
            << std::endl;
  exit( 1 );
}


//////
Ngf::~Ngf()
{
#if DEBUG
  std::cerr << "Ngf::~Ngf()" << std::endl;
#endif
  
  // terminate
  term();
}


//////
Ngf&
Ngf::operator=( const Ngf& n )
{
  std::cerr << "Ngf::operator=() : You cannot copy an NGF object." 
            << std::endl;
  exit( 1 );
  return( *this );
}


//////
bool
Ngf::init( unsigned int baseaddr, int debugLevel )
{
#if DEBUG
  std::cerr << "Ngf::init()" << std::endl;
#endif

  m_baseaddr   = baseaddr;
  m_debugLevel = debugLevel;

  // open device
  m_fd32 = open( "/dev/vme32d32",O_RDWR );
  if ( m_fd32 < 0 ){
    perror( "Ngf::init() : open /dev/vme32d32 :" );
    return( false );
  }
  
  // mmap
  m_map = (unsigned int*)mmap( 0,ngfmapsize,
                               PROT_WRITE|PROT_READ,MAP_SHARED,
                               m_fd32,m_baseaddr );
  if ( m_debugLevel || DEBUG ){
    std::cout << "Ngf::init() : map=" << m_map 
              << " baseaddr=0x" 
              << std::hex << std::setw(8) << std::setfill('0') 
              << m_baseaddr
              << std::dec << std::setw(0)
              << std::endl;
  }
  if ( (caddr_t)m_map == (caddr_t)-1 ){
    perror( "Ngf::init() : mmap " );
    return( false );
  }

  // VME slave memory
  m_fdslave = open( "/dev/vmedvma32",O_RDWR );
  if ( m_fdslave < 0 ){
    perror( "Ngf::init() : open /dev/vmedvma32" );
    return( -1 );
  }
  
  m_dvma.vme_addr  = 0;
  m_dvma.data_size = SFI_DMA_SIZE;
  m_dvma.bt        = VME_BT_D32|VME_BT_BLT|VME_BT_USER|VME_BT_WP;
  m_dvma.prot      = PROT_READ|PROT_WRITE;
  m_dvma.flags     = SMEM_VADDR;
  
  m_slaveptr = (caddr_t)vui_slave_map( m_fdslave,&m_dvma );
  if ( m_debugLevel || DEBUG ){
    std::cout << "slaveptr = 0x" << (unsigned int*)m_slaveptr << std::endl;
  }
  if( m_slaveptr == 0 ){
    perror( "Ngf::init() : vui_slave_map" );
    exit(1);
  } 
 
  m_slaveaddr = (unsigned int)m_dvma.vme_addr;
  if ( m_debugLevel || DEBUG ){
    std::cout << "slaveaddr = 0x" << m_slaveaddr << std::endl;
  }

  //
  return( true );
}


//////
bool
Ngf::term()
{
#if DEBUG
  std::cerr << "Ngf::term()" << std::endl;
#endif

  // VME slave memory
  vui_slave_unmap( m_fdslave, &m_dvma, m_slaveptr );
  if ( m_fdslave >= 0 ){
    close( m_fdslave );
  }

  // mmap
  if ( (caddr_t)m_map != (caddr_t)-1 ){
    munmap( (char*)m_map,ngfmapsize );
  }
  if ( m_fd32 >= 0 ){
    close( m_fd32 );
  }


  return( true );
}


//////
void
Ngf::command( unsigned int reg, unsigned int cmd )
{
#if DEBUG
  std::cerr << "Ngf::command()" << std::endl;
#endif

  if ( m_debugLevel || DEBUG )
    std::cout << "Ngf::command : reg=0x" 
              << std::hex << std::setw(8) << std::setfill('0') 
              << reg
              << " cmd=0x" << std::setw(8) << std::setfill('0')
              << cmd 
              << " addr=0x" << std::setw(8) << std::setfill('0')
              << NGF_MAP_INDEX(reg)
              << std::dec << std::setw(0)
              << std::endl;
  m_map[NGF_MAP_INDEX(reg)] = cmd;
}


//////
unsigned int
Ngf::status( unsigned int reg )
{
#if DEBUG
  std::cerr << "Ngf::status()" << std::endl;
#endif

  unsigned int  stat = m_map[NGF_MAP_INDEX(reg)];
  if ( m_debugLevel || DEBUG )
    std::cout << "Ngf::status : reg=0x" 
              << std::hex << std::setw(8) << std::setfill('0')
              << reg
              << " addr=0x" << std::setw(8) << std::setfill('0')
              << NGF_MAP_INDEX(reg)
              << " stat=0x"  << std::setw(8) << std::setfill('0')
              << stat 
              << std::dec << std::setw(0) 
              << std::endl;
  return( stat );
}


//////
unsigned int
Ngf::waitSeqDoneR( unsigned int* ret )
{
#if DEBUG
  std::cerr << "Ngf::waitSeqDoneR()" << std::endl;
#endif

  unsigned int  stat = 0;
  for( int i=0;i<1000;i++ ){
    stat = SeqStatusReg();
#if 0 && DEBUG
    std::cout << "stat=" 
              << std::hex << std::setw(8) << std::setfill('0')
              << stat 
              << std::dec << std::setw(0) << std::endl;
#endif
    if ( (stat & 0x8001) == 0x8001 ||  // FIFO mode
         (stat & 0x4f02) == 0x4802  ){ // LIST mode
      for( int j=0;j<100;j++ ){
        stat = SeqFifoAndEclNimInputReg();
        *ret = Seq2VmeFifo( 0 );
#if DEBUG
        std::cout << " stat=" 
                  << std::hex << std::setw(8) << std::setfill('0')
                  << stat 
                  << std::dec << std::setw(0) << std::endl;
#endif
        if ( !(stat & 0x10) ){
          return( FbStatusReg2() );
        }
      }
      break; // timeout
    }
  }
  return( 0xffffffff );
}


//////
unsigned int
Ngf::waitSeqDoneW()
{
#if DEBUG
  std::cerr << "Ngf::waitSeqDoneW()" << std::endl;
#endif

  unsigned int  stat = 0;
  for( int i=0;i<1000;i++ ){
    stat = SeqStatusReg();
#if DEBUG
    std::cout << "trial=" << i << " stat=" 
              << std::hex << std::setw(8) << std::setfill('0')
              << stat 
              << std::dec << std::setw(0) << std::endl;
#endif
    if ( (stat & 0x8001) == 0x8001 || // FIFO mode
         (stat & 0x8004) == 0x8004 || // RAM load mode
         (stat & 0x4f01) == 0x4801  ) // LIST mode
      return( FbStatusReg2() );
  }
  
  return( 0xffffffff );
}


// Read Sequencer FIFO
unsigned int
Ngf::readFifo( int size, unsigned int* data )
{
#if DEBUG
  std::cerr << "Ngf::readFifo()" << std::endl;
#endif

  unsigned int  stat,dt;
  for( int i=0; i<size; i++ ){
    for( int j=0;j<1000;j++ ){
      stat = SeqFifoAndEclNimInputReg();
      dt = Seq2VmeFifo( 0 );
      if ( !(stat & 0x10 ) )
        break;
    }
    data[i] = dt;
  }
  
  return( SeqStatusReg() );
}


// Read Sequencer FIFO
unsigned int
Ngf::readFifoNoWait( int size, unsigned int* data )
{
#if DEBUG
  std::cerr << "Ngf::readFifoNoWait()" << std::endl;
#endif

  if ( size > 0 ){
    unsigned int  stat,dt;
    // Wait until FIFO ready
    for( int i=0;i<1000;i++ ){
      stat = SeqFifoAndEclNimInputReg();
      dt = Seq2VmeFifo( 0 );
      if ( !(stat & 0x10 ) )
        break;
    }
    if ( stat & 0x10 )
      std::cerr << "Ngf::readFifoNoWait() : FIFO empty" << std::endl;
      
    data[0] = dt;
    for( int i=1; i<size; i++ ){
      data[i] = Seq2VmeFifo( 0 );
    }
  }
  
  return( SeqStatusReg() );
}


// List
unsigned int
Ngf::setList( unsigned int addr, const std::list<NgfList>& ls )
{
#if DEBUG
  std::cerr << "Ngf::SetList" << std::endl;
#endif
  
  SeqResetReg( 0 );
  SeqDisableReg( 0 );
  NextSeqRamAddressReg( addr );
  SeqRamLoadEnableReg( 0 );
  
  for( std::list<NgfList>::const_iterator i = ls.begin();
       i != ls.end(); i++ ){
    Vme2SeqFifo( i->func, i->data );
  }
  waitSeqDoneW();

  SeqRamLoadDisableReg( 0 );
  SeqResetReg( 0 );
  SeqEnableReg( 0 );
 
  return( NextSeqRamAddressReg() );
}


// List
unsigned int
Ngf::enableList()
{
#if DEBUG
  std::cerr << "Ngf::EnableList" << std::endl;
#endif
  
  SeqResetReg( 0 );
  SeqEnableReg( 0 );
  Vme2SeqFifo( NGF_ENABLE_RAM_SEQUENCER,0 );

  return( waitSeqDoneW() );
}


// List
unsigned int
Ngf::disableList()
{
#if DEBUG
  std::cerr << "Ngf::DisableList" << std::endl;
#endif
  
  SeqResetReg( 0 );
  SeqEnableReg( 0 );
  Vme2SeqFifo( NGF_DISABLE_RAM_SEQUENCER,0 );

  return( waitSeqDoneW() );
}


// Fastbus action
unsigned int
Ngf::frc( int pa,int sa, unsigned int* ret )
{
#if DEBUG
  std::cerr << "Ngf::frc()" << std::endl;
#endif

  Vme2SeqFifo( FB_PRIM_CSR,   pa );
  Vme2SeqFifo( FB_SECAD_W,    sa );
  Vme2SeqFifo( FB_DATA_R_DIS, 0  );
  
  return( waitSeqDoneR( ret ) );
}


unsigned int
Ngf::fwc( int pa,int sa, unsigned int data )
{
#if DEBUG
  std::cerr << "Ngf::fwc()" << std::endl;
#endif

  Vme2SeqFifo( FB_PRIM_CSR,   pa   );
  Vme2SeqFifo( FB_SECAD_W,    sa   );
  Vme2SeqFifo( FB_DATA_W_DIS, data );

  return( waitSeqDoneW() );
}


unsigned int
Ngf::frd( int pa,int sa, unsigned int* ret )
{
#if DEBUG
  std::cerr << "Ngf::frd()" << std::endl;
#endif

  Vme2SeqFifo( FB_PRIM_DSR,   pa );
  Vme2SeqFifo( FB_SECAD_W,    sa );
  Vme2SeqFifo( FB_DATA_R_DIS, 0  );

  return( waitSeqDoneR( ret ) );
}


unsigned int
Ngf::fwd( int pa,int sa, unsigned int data )
{
#if DEBUG
  std::cerr << "Ngf::fwd()" << std::endl;
#endif

  Vme2SeqFifo( FB_PRIM_DSR,   pa   );
  Vme2SeqFifo( FB_SECAD_W,    sa   );
  Vme2SeqFifo( FB_DATA_W_DIS, data );

  return( waitSeqDoneW() );
}


unsigned int
Ngf::frdb( int pa, int sa, unsigned int mode, 
           unsigned int limitcounter, unsigned int buf, 
           unsigned int* recword, unsigned int* nextbuf )
{
  // buf : Address of LWord buffer (VME-Slave address)
  // mode :
  // 0x10 Only Direct
  // 0x09 VMED32 data cycle
  // 0x0a VMED32 block transfer
  // 0x19 (0x10|0x09)
  // 0x1a (0x10|0x0a)

#if DEBUG
  std::cerr << "Ngf::frdb()" << std::endl;
#endif
  unsigned int  maxwd = ((limitcounter-1) & 0x00ffffff) | ((mode & 0xff)<<24);
#if DEBUG
  std::cerr << "Ngf::frdb : maxwd=" 
            << std::hex << std::setw(8) << std::setfill('0')
            << maxwd
            << std::dec << std::setw(0) << std::endl;
#endif
  
  Vme2SeqFifo( FB_PRIM_DSR,      pa    );
  Vme2SeqFifo( FB_SECAD_W,       sa    );
  Vme2SeqFifo( FB_LD_BLT_AP,     buf   );
  Vme2SeqFifo( FB_START_FRDB,    maxwd );
  Vme2SeqFifo( FB_DISCON,        0     );
  Vme2SeqFifo( FB_STORE_FRDB_WC, 0     );
  Vme2SeqFifo( FB_STORE_FRDB_AP, 0     );

  unsigned int  rdata  = 0;
  unsigned int  status = waitSeqDoneR( &rdata );
  *recword = rdata & 0x00ffffff;
#if DEBUG
  std::cout << "Ngf::frdb: status=0x" 
            << std::hex <<std::setw(8) << std::setfill('0')
            << status
            << std::dec << std::setw(0) << std::endl;
#endif

  waitSeqDoneR( nextbuf );
#if DEBUG
  std::cout << "Ngf::frdb: nextbuf="
            << std::hex << std::setw(8) << std::setfill('0')
            << *nextbuf
            << std::dec << std::setw(0) << std::endl;
#endif

  return( status );
}


unsigned int
Ngf::frdbm( int pa, int sa, unsigned int mode, 
            unsigned int limitcounter, unsigned int buf, 
            unsigned int* recword, unsigned int* nextbuf )
{
  // buf : Address of LWord buffer (VME-Slave address)
  // mode :
  // 0x10 Only Direct
  // 0x09 VMED32 data cycle
  // 0x0a VMED32 block transfer
  // 0x19 (0x10|0x09)
  // 0x1a (0x10|0x0a)

#if DEBUG
  std::cerr << "Ngf::frdbm()" << std::endl;
#endif
  unsigned int  maxwd = ((limitcounter-1) & 0x00ffffff) | ((mode & 0xff)<<24);
#if DEBUG
  std::cerr << "Ngf::frdb : maxwd=" 
            << std::hex << std::setw(8) << std::setfill('0')
            << maxwd
            << std::dec << std::setw(0) << std::endl;
#endif

  Vme2SeqFifo( FB_PRIM_DSRM,     pa    );
  Vme2SeqFifo( FB_SECAD_W,       sa    );
  Vme2SeqFifo( FB_LD_BLT_AP,     buf   );
  Vme2SeqFifo( FB_START_FRDB,    maxwd );
  Vme2SeqFifo( FB_DISCON_RM,     0     );
  Vme2SeqFifo( FB_STORE_FRDB_WC, 0     );
  Vme2SeqFifo( FB_STORE_FRDB_AP, 0     );

  unsigned int  rdata  = 0;
  unsigned int  status = waitSeqDoneR( &rdata );
  *recword = rdata & 0x00ffffff;
#if DEBUG
  std::cout << "Ngf::frdb: status=0x" 
            << std::hex <<std::setw(8) << std::setfill('0')
            << *recword 
            << std::dec << std::setw(0) << std::endl;
#endif

  waitSeqDoneR( nextbuf );
#if DEBUG
  std::cout << "Ngf::frdb: nextbuf="
            << std::hex << std::setw(8) << std::setfill('0')
            << *nextbuf
            << std::dec << std::setw(0) << std::endl;
#endif

  return( status );
}


unsigned int
Ngf::frdb_single( int pa, int n_sa, int sa[], unsigned int data[] )
{
#if DEBUG
  std::cerr << "Ngf::frdb_single()" << std::endl;
#endif

  Vme2SeqFifo( FB_PRIM_DSR, pa );
  for( int i=0; i<n_sa; i++ ){
    Vme2SeqFifo( FB_SECAD_W, sa[i] );
    Vme2SeqFifo( FB_DATA_R,  0     );
  }
  Vme2SeqFifo( FB_DISCON, 0 );

  for( int i=0; i<n_sa; i++ ){
    waitSeqDoneR( &data[i] );
  }

  return( SeqStatusReg() );
}


unsigned int
Ngf::frdb_single( int pa, int n_sa, unsigned int data[] )
{
#if DEBUG
  std::cerr << "Ngf::frdb_single()" << std::endl;
#endif
  
  Vme2SeqFifo( FB_PRIM_DSR, pa );
  for( int i=0; i<n_sa; i++ ){
    Vme2SeqFifo( FB_DATA_R, 0 );
  }
  Vme2SeqFifo( FB_DISCON, 0 );

  for( int i=0; i<n_sa; i++ ){
    waitSeqDoneR( &data[i] );
  }

  return( SeqStatusReg() );
}


