// -*- C++ -*-
//
// Smp.cc
// SMP-TKO accessing class
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Wed Nov 21 13:14:21 JST 2001
//
// $Id: Smp.cc,v 1.1.1.1 2002/08/13 12:48:12 kensh Exp $
//
// Revision History
//
// $Log: Smp.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <iomanip>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>
#include "smp/Smp.h"
#include <sys/vui.h>



//////
#define DEBUG 0


//////
const static int  smpmapsize = 0x00180100;


//////
Smp::Smp()
  : m_debugLevel( 0 ),
    m_baseaddr( 0 ),
    m_fd32( -1 ),
    m_fd32fdma( -1 ),
    m_map( (unsigned int*)-1 )
{
#if DEBUG
  std::cerr << "Smp::Smp()" << std::endl;
#endif
}


//////
Smp::Smp( unsigned int baseaddr, int debugLevel )
{
#if DEBUG
  std::cerr << "Smp::Smp(...)" << std::endl;
#endif

  if ( !init( baseaddr,debugLevel) ){
    std::cout << "Smp::Smp() : init failed" << std::endl;
    exit( 1 );
  }
}


////// 
Smp::Smp( const Smp& s )
{
#if DEBUG
  std::cerr << "Smp::Smp(copy)" << std::endl;
#endif

  term();

  if ( !init( s.m_baseaddr,s.m_debugLevel ) ){
    std::cout << "Smp::Smp(copy) : init failed" << std::endl;
    exit( 1 );
  }
}

  
//////
Smp::~Smp()
{
#if DEBUG
  std::cerr << "Smp::term()" << std::endl;
#endif
  
  term();
}


//////
Smp&
Smp::operator=( const Smp& s )
{
  term();

  if ( !init( s.m_baseaddr,s.m_debugLevel ) ){
    std::cout << "Smp::operator=() : init failed" << std::endl;
    exit( 1 );
  }

  return( *this );
}


//////
void
Smp::tkoCommand( int reg,int cmd )
{
  if ( m_debugLevel )
    std::cout << "Smp::tkocmd : reg=0x" 
              << std::hex << std::setw(8) << std::setfill('0') 
              << reg
              << " cmd=0x" << std::setw(8) << std::setfill('0')
              << cmd 
              << " addr=0x" << std::setw(8) << std::setfill('0')
              << SMP_REG_INDEX(reg)
              << std::dec
              << std::endl;
  m_map[SMP_REG_INDEX(reg)] = cmd;
}


//////
unsigned int
Smp::tkoStatus( int reg )
{
  unsigned int  stat = m_map[SMP_REG_INDEX(reg)];
  if ( m_debugLevel )
    std::cout << "Smp::tkoStatus : reg=0x" 
              << std::hex << std::setw(8) << std::setfill('0')
              << reg
              << " addr=0x" << std::setw(8) << std::setfill('0')
              << SMP_REG_INDEX(reg)
              << " stat=0x"  << std::setw(8) << std::setfill('0')
              << stat 
              << std::dec
              << std::endl;
  return( stat );
}


//////
unsigned int
Smp::tkoRead(  int MA,int SA )
{
  unsigned int  addr = SMP_TKOSPACE | SMP_TKOMA(MA) | SMP_TKOSA(SA);
  unsigned int  data = m_map[SMP_ADDR_INDEX(addr)];

  if ( m_debugLevel )
    std::cout << "Smp::tkoRead : addr=0x" 
              << std::hex << std::setw(8) << std::setfill('0')
              << addr
              << " data=0x" << std::setw(8) << std::setfill('0')
              << data
              << std::dec 
              << std::endl;

  return( data );
}


//////
int
Smp::tkoReadB( int MA, int SA, int size, unsigned int* data )
{
  for( int i=0;i<size;i++ ){
    unsigned int  addr = SMP_TKOSPACE | SMP_TKOMA(MA) | SMP_TKOSA(SA+i);
    data[i] = m_map[SMP_ADDR_INDEX(addr)];
    
    if ( m_debugLevel )
      std::cout << "Smp::tkoReadB : addr=0x" 
                << std::hex << std::setw(8) << std::setfill('0')
                << addr
                << " ma=" << std::dec << MA
                << " sa=" << SA+i
                << " data=0x" 
                << std::hex << std::setw(8) << std::setfill('0') 
                << data[SA+i]
                << std::dec 
                << std::endl;
  }
  return( size );
}


//////
void
Smp::tkoWrite( int MA, int SA, int BC, unsigned int data )
{
  unsigned int  addr = SMP_TKOSPACE|SMP_TKOBC(BC)|SMP_TKOMA(MA)|SMP_TKOSA(SA);
  m_map[SMP_ADDR_INDEX(addr)] = data;
  
  if ( m_debugLevel )
    std::cout << "Smp::tkoWrite : addr=0x" 
              << std::hex << std::setw(8) << std::setfill('0')
              << addr
              << " data=0x" << std::setw(8) << std::setfill('0')
              << data
              << std::dec 
              << std::endl;
}


//////
unsigned int
Smp::smpRead( unsigned int index )
{
  unsigned int  data = m_map[index];

  if ( m_debugLevel )
    std::cout << "Smp::smpRead : index=0x"
              << std::hex << std::setw(8) << std::setfill('0')
              << index
              << " data=0x" << std::setw(8) << std::setfill('0')
              << data
              << std::dec 
              << std::endl;

  return( data );
}

//////
int
Smp::smpReadB( unsigned int index, int size, unsigned int* data )
{
  //memcpy( data,&m_map[index],size );
  //std::cout << "&data =" << data << std::endl;
  //std::cout << "&m_map=" << &m_map[index] << std::endl;
  int  s = size >> 2;
  for( int i=0; i<s; i++ )
    data[i] = m_map[index+i];
  
  if ( m_debugLevel ){
    for( int i=0; i<s; i++ )
      std::cout << "Smp::smpReadB : index=0x"
                << std::hex << std::setw(8) << std::setfill('0')
                << index+i
                << " data=0x" << std::setw(8) << std::setfill('0')
                << data[i]
                << std::dec 
                << std::endl;
  }
  return( size );
}


//////
int
Smp::smpReadD( unsigned int index, int size )
{
  // from spec_fdma_read.c
  if( lseek( m_fd32fdma,m_baseaddr,SEEK_SET ) == -1 ){
    perror( "Smp::smpReadD : lseek " );
    exit(1);
  }

  if( ( size = read( m_fd32fdma, (void*)m_fdma.ioaddr, size ) ) == -1 ){
    perror( "Smp::smpReadD : read " );
    exit(1);
  }
  
  if ( m_debugLevel ){
    int  s = size >> 2;
    unsigned int*  data = (unsigned int*)m_fdmaptr;
    for( int i=0; i<s; i++ )
      std::cout << "Smp::smpReadD : index=0x"
                << std::hex << std::setw(8) << std::setfill('0')
                << index+i
                << " data=0x" << std::setw(8) << std::setfill('0')
                << data[i]
                << std::dec 
                << std::endl;
  }

  return( size );
}


//////
void
Smp::bufferSwitch( int mask )
{
  tkoCommand( SMP_BCR,SMP_BCR_SOFTSWITCH | mask );
}


//////
bool
Smp::bufferSwitchWait( int timeout_count )
{
  // check flag
  int  ct = timeout_count;
  while( ct-- ){
    if ( !(tkoStatus( SMP_BCR ) & SMP_BCR_SWFLAG ) ){
      if ( m_debugLevel ){
        std::cout << "bufferSwitchWait : count=" << timeout_count - ct
                  << "  " << std::endl;
      }
      return( false );
    }
  }
  return( true );
}


//////
bool
Smp::busy()
{
  return( (tkoStatus( SMP_STAR ) & SMP_STAR_BUSY) != 0 );
}


//////
bool
Smp::busyWait( int timeout_count )
{
  // check BUSY
  int  ct = timeout_count;
  while( ct-- ){
    if ( !(tkoStatus( SMP_STAR ) & SMP_STAR_BUSY ) ){
      if ( m_debugLevel ){
        std::cout << "busyWait : count=" << timeout_count - ct
                  << "  " << std::endl;
      }
      return( false );
    }
  }
  return( true );
}


//////
bool
Smp::init( unsigned int baseaddr, int debugLevel )
{
  m_baseaddr   = baseaddr;
  m_debugLevel = debugLevel;

  // open device
  m_fd32 = open( "/dev/vme32d32",O_RDWR );
  if ( m_fd32 < 0 ){
    perror( "Smp::init() : /dev/vme32d32 " );
    return( false );
  }
  
  // mmap
  m_map = (unsigned int*)mmap( 0,smpmapsize,
                               PROT_WRITE|PROT_READ,MAP_SHARED,
                               m_fd32,m_baseaddr );
  if ( m_debugLevel ){
    std::cout << "Smp::init() : map=0x" << m_map << std::endl;
  }
  if ( (caddr_t)m_map == (caddr_t)-1 ){
    perror( "Smp::init() : mmap " );
    return( false );
  }

  // open fdma device
  m_fd32fdma = open( "/dev/vmefdma32blt",O_RDWR );
  if ( m_fd32fdma < 0 ){
    perror( "Smp::init() : /dev/vmefdma32blt " );
    return( false );
  }

  // init DMA
  m_fdma.data_size = 0x00100000; // 1MB
  m_fdma.vme_addr  = m_baseaddr;
  m_fdma.kvaddr    = (caddr_t)0;
  m_fdma.bt        = (bt_t)0;
  m_fdma.prot      = (int)0;
  m_fdma.flags     = (uint_t)SMEM_VADDR;
  m_fdmaptr = (char*)vui_fdma_malloc( m_fd32fdma,&m_fdma );

  //
  return( true );
}



//////
bool
Smp::term()
{
  if ( (caddr_t)m_map != (caddr_t)-1 ){
    munmap( (char*)m_map,smpmapsize );
  }

  if ( m_fd32 >= 0 ){
    close( m_fd32 );
  }

  if ( m_fd32fdma >= 0 ){
    vui_fdma_free( m_fd32fdma, &m_fdma, m_fdmaptr );
    close( m_fd32fdma );
  }

  return( true );
}


