// -*- C++ -*-
//
// gustep.cc
// C++ interface of GUSTEP
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Wed Jul 11 15:29:57 JST 2001
//
// $Id: Gustep.cc,v 1.1.1.1 2002/08/13 12:48:09 kensh Exp $
//
// Revision History
//
// $Log: Gustep.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:09  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <string>
#include <math.h>
#include "ccfortran/G321Common.h"
#include "ccfortran/ccfortran.h"
#include "gsim-general/GsimGeneral.h"


//////
#define  DEBUG  0


// GsimHits.cc
extern int  GsimHits();

// Gukine.cc
extern int  GenMechanism[];

// Gutrak.cc
extern bool  StopEvent;


//////
void
Gustep()
{
  if ( DEBUG || gcflag_.ISWIT[3-1] > 0 )
    std::cerr << "Gustep" << std::endl;
  
  // Check Volume
  int  stop = GsimHits();

  if ( stop ){
    if ( stop < 0 ){
      // Stop Event
      if ( DEBUG || gcflag_.ISWIT[3-1] > 0 ){
        std::cerr << "Gustep : Force stop event : EV=" << gcflag_.IEVENT
                  << " ITRA=" << gckine_.ITRA << " NTRACK=" << gcnum_.NTRACK 
                  << std::endl;
      }
      StopEvent = true; // see Gutrak.cc, Gukine.cc
      gctrak_.ISTOP = -stop;
    }
    else{
      // Stop swimming
      gctrak_.ISTOP = stop;
    }
  }
  else{
    // Store secondary particle
    for( int i=0;i<gcking_.NGKINE;i++ ){
      int  type = (int)gcking_.GKIN[i][4];
      if ( type == 4 ) // skip neutrino
        gcking_.IFLGK[i] = -1;
      else
        gcking_.IFLGK[i] = 1;
      GSKING( i+1 );
      // mechanism
      if ( 0 < gcking_.IFLGK[i] ){
        if ( gcking_.IFLGK[i] < MAX_MECH ){
          GenMechanism[gcking_.IFLGK[i]] = gcking_.KCASE;
        }
        else{
          static bool  first = true; 
          if ( first )
            std::cerr << "Gustep : #particle exceeds the size of mech[]\n"
                      << std::endl;
          first = false;
        }
      }
    }
  }
  if ( DEBUG || gcflag_.ISWIT[3-1] > 1 ){
    std::cerr << "Gustep : NGKINE=" << gcking_.NGKINE << std::endl;
  }

  //
  if ( Display ){ 
    if ( gcflag_.ISWIT[4-1] == 1 )
      GSXYZ();
    else if ( gcflag_.ISWIT[4-1] == 3 )
      GDCXYZ();
  }

}

