// -*- C++ -*-
//
// E391Detector.cc
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Aug  6 09:22:10 JST 2002
//
// $Id: E391Detector.cc,v 1.12 2002/12/04 10:02:06 yamaga Exp $
//
// Revision History
//
// $Log: E391Detector.cc,v $
// Revision 1.12  2002/12/04 10:02:06  yamaga
// Updates for MultihitTDC.
//
// Revision 1.11  2002/12/03 01:56:37  yamaga
// Fixed memory leak.
//
// Revision 1.10  2002/12/01 14:25:39  yamaga
// Updated for multihit TDC.
//
// Revision 1.9  2002/11/20 06:46:20  yamaga
// add pedestalDB
//
// Revision 1.8  2002/10/31 17:57:20  yamaga
// ChargedTrigger corrected.
//
// Revision 1.7  2002/10/31 09:51:25  yamaga
// Hodoscope and ChargedTrigger added.
//
// Revision 1.6  2002/10/18 12:32:44  yamaga
// N_xxx correctly counts the number of detector componets.
//
// Revision 1.5  2002/10/13 14:36:40  yamaga
// Do not show the database contents in getDB().
//
// Revision 1.4  2002/10/07 11:55:10  yamaga
// Debug message corrected.
//
// Revision 1.3  2002/09/27 12:12:55  yamaga
// Conform to eventDisplay.
//
// Revision 1.2  2002/08/29 13:43:43  yamaga
// pbank() modified to bank()
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <iomanip>
#include <stdio.h>
#include "midas.h"
#include "unpack/AdcEvent.h"
#include "unpack/TdcEvent.h"
#include "detparam/DetParam.h"
#include "E391Detector/E391Detector.h"

#define DEBUG  0


// static members
E391Detector*  E391Detector::s_e391det = 0;

const Detector  E391Detector::s_dummydet;
const AdcEvent  E391Detector::s_dummyAdcEvent;
const TdcEvent  E391Detector::s_dummyTdcEvent;
const MTdcEvent E391Detector::s_dummyMTdcEvent;


// singleton
E391Detector&
E391Detector::instance()
{
  //std::cout << "E391Detector::instance()" << std::endl;

  if ( s_e391det == 0 ){
    s_e391det = new E391Detector;
    if ( s_e391det == 0 ){ // failed
      std::cout << "E391Detector::instance() :"
                << " cannot create E391Detector object."
                << std::endl;
      perror( "new" );
      abort();
    }
  }

  return( *s_e391det );
}
  

// 
E391Detector::E391Detector()
  : m_detparam( 0 )
{
  if ( DEBUG )
    std::cout << "E391Detector::E391Detector()" << std::endl;

  //
  for( int i=0;i<MAX_NTDCCRATE;i++ )
    m_tdcCrate[i] = 0;
  for( int i=0;i<MAX_NADCCRATE;i++ )
    m_adcCrate[i] = 0;;
  for( int i=0;i<MAX_NMTDCCRATE;i++ )
    m_mtdcCrate[i] = 0;;

  //
  init_param();
}


//
E391Detector::~E391Detector()
{
  if ( DEBUG )
    std::cout << "E391Detector::~E391Detector()" << std::endl;

  delete_param();
}


//
void
E391Detector::init_param()

{ 
  if ( DEBUG )
    std::cout << "E391Detector::init_param()" << std::endl;

  if ( DEBUG ){
    std::cout << "E391Detector::init_param()" << std::endl
              << "N_C6  =" << N_C6 << std::endl
              << "N_FB  =" << N_FRONTBARREL << std::endl
              << "N_CC01=" << N_CC01 << std::endl
              << "N_CC02=" << N_CC02 << std::endl
              << "N_MB  =" << N_MAINBARREL << std::endl
              << "N_CV  =" << N_CHARGEDVETO << std::endl
              << "N_CSI =" << N_CSI << std::endl
              << "N_Sandwich=" << N_SANDWICH << std::endl
              << "N_CC03=" << N_CC03 << std::endl
              << "N_CC04=" << N_CC04 << std::endl
              << "N_CC05=" << N_CC05 << std::endl
              << "N_BeamCatcher=" << N_BEAMCATCHER << std::endl
              << "N_CosmicCounter=" << N_COSMICCOUNTER << std::endl
              << "N_XeMonitor=" << N_XEMONITOR << std::endl
              << "N_Hodoscope=" << N_HODOSCOPE << std::endl
              << "N_ChargedTrigger=" << N_CHARGEDTRIGGER << std::endl;
    
  }
  
  for( int i=0;i<N_C6;i++ )
    m_C6[i].setDB( 0 );
  for( int i=0;i<N_FRONTBARREL;i++ )
    m_FrontBarrel[i].setDB( 0 );
  for( int i=0;i<N_CC01;i++ )
    m_CC01[i].setDB( 0 );
  for( int i=0;i<N_CC02;i++ )
    m_CC02[i].setDB( 0 );
  for( int i=0;i<N_MAINBARREL;i++ )
    m_MainBarrel[i].setDB( 0 );
  for( int i=0;i<N_CHARGEDVETO;i++ )
    m_ChargedVeto[i].setDB( 0 );
  for( int i=0;i<N_CSI;i++ )
    m_CsI[i].setDB( 0 );
  for( int i=0;i<N_SANDWICH;i++ )
    m_Sandwich[i].setDB( 0 );
  for( int i=0;i<N_CC03;i++ )
    m_CC03[i].setDB( 0 );
  for( int i=0;i<N_CC04;i++ )
    m_CC04[i].setDB( 0 );
  for( int i=0;i<N_CC05;i++ )
    m_CC05[i].setDB( 0 );
  for( int i=0;i<N_BEAMCATCHER;i++ )
    m_BeamCatcher[i].setDB( 0 );
  for( int i=0;i<N_COSMICCOUNTER;i++ )
    m_CosmicCounter[i].setDB( 0 );
  for( int i=0;i<N_XEMONITOR;i++ )
    m_XeMonitor[i].setDB( 0 );
  for( int i=0;i<N_HODOSCOPE;i++ )
    m_Hodoscope[i].setDB( 0 );
  for( int i=0;i<N_CHARGEDTRIGGER;i++ )
    m_ChargedTrigger[i].setDB( 0 );

  for( int i=0;i<N_C6_SUM;i++ )
    m_C6_Sum[i].setDB( 0 );
  for( int i=0;i<N_FRONTBARREL_SUM;i++ )
    m_FrontBarrel_Sum[i].setDB( 0 );
  for( int i=0;i<N_CC01_SUM;i++ )
    m_CC01_Sum[i].setDB( 0 );
  for( int i=0;i<N_CC02_SUM;i++ )
    m_CC02_Sum[i].setDB( 0 );
  for( int i=0;i<N_MAINBARREL_SUM;i++ )
    m_MainBarrel_Sum[i].setDB( 0 );
  for( int i=0;i<N_CHARGEDVETO_SUM;i++ )
    m_ChargedVeto_Sum[i].setDB( 0 );
  for( int i=0;i<N_CSI_SUM;i++ )
    m_CsI_Sum[i].setDB( 0 );
  for( int i=0;i<N_SANDWICH_SUM;i++ )
    m_Sandwich_Sum[i].setDB( 0 );
  for( int i=0;i<N_CC03_SUM;i++ )
    m_CC03_Sum[i].setDB( 0 );
  for( int i=0;i<N_CC04_SUM;i++ )
    m_CC04_Sum[i].setDB( 0 );
  for( int i=0;i<N_CC05_SUM;i++ )
    m_CC05_Sum[i].setDB( 0 );
  for( int i=0;i<N_BEAMCATCHER_SUM;i++ )
    m_BeamCatcher_Sum[i].setDB( 0 );
  for( int i=0;i<N_COSMICCOUNTER_SUM;i++ )
    m_CosmicCounter_Sum[i].setDB( 0 );
  for( int i=0;i<N_XEMONITOR_SUM;i++ )
    m_XeMonitor_Sum[i].setDB( 0 );
  for( int i=0;i<N_HODOSCOPE_SUM;i++ )
    m_Hodoscope_Sum[i].setDB( 0 );
  for( int i=0;i<N_CHARGEDTRIGGER_SUM;i++ )
    m_ChargedTrigger_Sum[i].setDB( 0 );

  
  m_pheader = 0;
  m_pevent  = 0;

  N_C6 = 0;
  N_FRONTBARREL = 0;
  N_CC01 = 0;
  N_CC02 = 0;
  N_MAINBARREL = 0;
  N_CHARGEDVETO = 0;
  N_CSI = 0;
  N_SANDWICH = 0;
  N_CC03 = 0;
  N_CC04 = 0;
  N_CC05 = 0;
  N_BEAMCATCHER = 0;
  N_COSMICCOUNTER = 0;
  N_XEMONITOR = 0;
  N_HODOSCOPE = 0;
  N_CHARGEDTRIGGER = 0;

  N_C6_SUM = 0;
  N_FRONTBARREL_SUM = 0;
  N_CC01_SUM = 0;
  N_CC02_SUM = 0;
  N_MAINBARREL_SUM = 0;
  N_CHARGEDVETO_SUM = 0;
  N_CSI_SUM = 0;
  N_SANDWICH_SUM = 0;
  N_CC03_SUM = 0;
  N_CC04_SUM = 0;
  N_CC05_SUM = 0;
  N_BEAMCATCHER_SUM = 0;
  N_COSMICCOUNTER_SUM = 0;
  N_XEMONITOR_SUM = 0;
  N_HODOSCOPE_SUM = 0;
  N_CHARGEDTRIGGER_SUM = 0;
  
}


//
void
E391Detector::delete_param()
{
  if ( DEBUG )
    std::cout << "E391Detector::delete_param()" << std::endl;
  
  for( int i=0;i<MAX_NTDCCRATE;i++ ){
    delete m_tdcCrate[i];
    m_tdcCrate[i] = 0;
  }
  
  for( int i=0;i<MAX_NADCCRATE;i++ ){
    delete m_adcCrate[i];
    m_adcCrate[i] = 0;
  }

  for( int i=0;i<MAX_NMTDCCRATE;i++ ){
    delete m_mtdcCrate[i];
    m_mtdcCrate[i] = 0;
  }

}


//
int
E391Detector::getDB( int runnum )
{
  if ( DEBUG )
    std::cout << "E391Detector::getDB()" << std::endl;
  
  if ( runnum <= 0 )
    runnum = m_runNumber;

  // 
  m_detparam = &DetParam::instance();
  m_detparam->getDB( runnum );
  m_detparam->getPedDB( runnum );

  // clear numbers
  delete_param();
  init_param();

  // Detector channels setup
  for( std::list<E391_DETE_DB>::const_iterator 
         p = m_detparam->paramList().begin();
       p!=m_detparam->paramList().end(); p++ ){
    
    if ( p->name == "CSI" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CSI ){
        //std::cout << " CsI ID=" << p->ID << std::endl;
        if ( m_CsI[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CsI#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CsI[ p->ID ].setDB( &(*p) );
          // N_CSI++;
          if ( N_CSI <= p->ID )
            N_CSI = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CV" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CHARGEDVETO ){
        //std::cout << " CV   ID=" << p->ID << std::endl;
        if ( m_ChargedVeto[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for ChargedVeto#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_ChargedVeto[ p->ID ].setDB( &(*p) );
          //N_CHARGEDVETO++;
          if ( N_CHARGEDVETO <= p->ID )
            N_CHARGEDVETO = p->ID + 1;
        }
      }
    }
    else if ( p->name == "SANDWICH" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_SANDWICH ){
        //std::cout << " SAND ID=" << p->ID << std::endl;
        if ( m_Sandwich[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for Sandwich#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_Sandwich[ p->ID ].setDB( &(*p) );
          //N_SANDWICH++;
          if ( N_SANDWICH <= p->ID )
            N_SANDWICH = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CC03" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CC03 ){
        //std::cout << " CC03 ID=" << p->ID << std::endl;
        if ( m_CC03[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CC03#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CC03[ p->ID ].setDB( &(*p) );
          //N_CC03++;
          if ( N_CC03 <= p->ID )
            N_CC03 = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CC04" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CC04 ){
        //std::cout << " CC04 ID=" << p->ID << std::endl;
        if ( m_CC04[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CC04#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CC04[ p->ID ].setDB( &(*p) );
          //N_CC04++;
          if ( N_CC04 <= p->ID )
            N_CC04 = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CC05" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CC05 ){
        //std::cout << " CC05 ID=" << p->ID << std::endl;
        if ( m_CC05[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CC05#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CC05[ p->ID ].setDB( &(*p) );
          //N_CC05++;
          if ( N_CC05 <= p->ID )
            N_CC05 = p->ID + 1;
        }
      }
    }
    else if ( p->name == "BC" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_BEAMCATCHER ){
        //std::cout << " BC   ID=" << p->ID << std::endl;
        if ( m_BeamCatcher[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for BeamCatcher#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_BeamCatcher[ p->ID ].setDB( &(*p) );
          //N_BEAMCATCHER++;
          if ( N_BEAMCATCHER <= p->ID )
            N_BEAMCATCHER = p->ID + 1;
        }
      }
    }
    else if ( p->name == "C6" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_C6 ){
        //std::cout << " C6   ID=" << p->ID << std::endl;
        if ( m_C6[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for C6#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_C6[ p->ID ].setDB( &(*p) );
          //N_C6++;
          if ( N_C6 <= p->ID )
            N_C6 = p->ID + 1;
        }
      }
    }
    else if ( p->name == "FRONTBARREL" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_FRONTBARREL ){
        //std::cout << " FBAR ID=" << p->ID << std::endl;
        if ( m_FrontBarrel[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for FrontBarrel#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_FrontBarrel[ p->ID ].setDB( &(*p) );
          //N_FRONTBARREL++;
          if ( N_FRONTBARREL <= p->ID )
            N_FRONTBARREL = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CC01" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CC01 ){
        //std::cout << " CC01 ID=" << p->ID << std::endl;
        if ( m_CC01[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CC01#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CC01[ p->ID ].setDB( &(*p) );
          //N_CC01++;
          if ( N_CC01 <= p->ID )
            N_CC01 = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CC02" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CC02 ){
        //std::cout << " CC02 ID=" << p->ID << std::endl;
        if ( m_CC02[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CC02#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CC02[ p->ID ].setDB( &(*p) );
          //N_CC02++;
          if ( N_CC02 <= p->ID )
            N_CC02 = p->ID + 1;
        }
      }
    }
    else if ( p->name == "MAINBARREL" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_MAINBARREL ){
        //std::cout << " MBAR ID=" << p->ID << std::endl;
        if ( m_MainBarrel[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for MainBarrel#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_MainBarrel[ p->ID ].setDB( &(*p) );
          //N_MAINBARREL++;
          if ( N_MAINBARREL <= p->ID )
            N_MAINBARREL = p->ID + 1;
        }
      }
    }
    else if ( p->name == "COSMIC" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_COSMICCOUNTER ){
        //std::cout << " COSM ID=" << p->ID << std::endl;
        if ( m_CosmicCounter[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CosmicCounter#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CosmicCounter[ p->ID ].setDB( &(*p) );
          //N_COSMICCOUNTER++;
          if ( N_COSMICCOUNTER <= p->ID )
            N_COSMICCOUNTER = p->ID + 1;
        }
      }
    }
    else if ( p->name == "XEMON" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_XEMONITOR ){
        //std::cout << " XE ID=" << p->ID << std::endl;
        if ( m_XeMonitor[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for XeMonitor#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_XeMonitor[ p->ID ].setDB( &(*p) );
          //N_XEMONITOR++;
          if ( N_XEMONITOR <= p->ID )
            N_XEMONITOR = p->ID + 1;
        }
      }
    }
    else if ( p->name == "HODOSCOPE" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_HODOSCOPE ){
        //std::cout << " HODOSCOPE ID=" << p->ID << std::endl;
        if ( m_Hodoscope[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for Hodoscope#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_Hodoscope[ p->ID ].setDB( &(*p) );
          //N_HODOSCOPE++;
          if ( N_HODOSCOPE <= p->ID )
            N_HODOSCOPE = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CVTRG" && p->type == "EACH" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CHARGEDTRIGGER ){
        //std::cout << " CVTRG ID=" << p->ID << std::endl;
        if ( m_ChargedTrigger[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for ChargedTrigger#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_ChargedTrigger[ p->ID ].setDB( &(*p) );
          //N_CHARGEDTRIGGER++;
          if ( N_CHARGEDTRIGGER <= p->ID )
            N_CHARGEDTRIGGER = p->ID + 1;
        }
      }
    }

    // SUM
    else if ( p->name == "CSI" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CSI_SUM ){
        //std::cout << " CsI Sum ID=" << p->ID << std::endl;
        if ( m_CsI_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CsI_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CsI_Sum[ p->ID ].setDB( &(*p) );
          //N_CSI_SUM++;
          if ( N_CSI_SUM <= p->ID )
            N_CSI_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CV" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CHARGEDVETO_SUM ){
        //std::cout << " CV Sum   ID=" << p->ID << std::endl;
        if ( m_ChargedVeto_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for ChargedVeto_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_ChargedVeto_Sum[ p->ID ].setDB( &(*p) );
          //N_CHARGEDVETO_SUM++;
          if ( N_CHARGEDVETO_SUM <= p->ID )
            N_CHARGEDVETO_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "SANDWICH" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_SANDWICH_SUM ){
        //std::cout << " SAND Sum ID=" << p->ID << std::endl;
        if ( m_Sandwich_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for Sandwich_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_Sandwich_Sum[ p->ID ].setDB( &(*p) );
          //N_SANDWICH_SUM++;
          if ( N_SANDWICH_SUM <= p->ID )
            N_SANDWICH_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CC03" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CC03_SUM ){
        //std::cout << " CC03 Sum ID=" << p->ID << std::endl;
        if ( m_CC03_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CC03_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CC03_Sum[ p->ID ].setDB( &(*p) );
          //N_CC03_SUM++;
          if ( N_CC03_SUM <= p->ID )
            N_CC03_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CC04" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CC04_SUM ){
        //std::cout << " CC04 Sum ID=" << p->ID << std::endl;
        if ( m_CC04_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CC04_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CC04_Sum[ p->ID ].setDB( &(*p) );
          //N_CC04_SUM++;
          if ( N_CC04_SUM <= p->ID )
            N_CC04_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CC05" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CC05_SUM ){
        //std::cout << " CC05 Sum ID=" << p->ID << std::endl;
        if ( m_CC05_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CC05_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CC05_Sum[ p->ID ].setDB( &(*p) );
          //N_CC05_SUM++;
          if ( N_CC05_SUM <= p->ID )
            N_CC05_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "BC" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_BEAMCATCHER_SUM ){
        //std::cout << " BC Sum   ID=" << p->ID << std::endl;
        if ( m_BeamCatcher_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for BeamCatcher_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_BeamCatcher_Sum[ p->ID ].setDB( &(*p) );
          //N_BEAMCATCHER_SUM++;
          if ( N_BEAMCATCHER_SUM <= p->ID )
            N_BEAMCATCHER_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "C6" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_C6_SUM ){
        //std::cout << " C6 Sum   ID=" << p->ID << std::endl;
        if ( m_C6_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for C6_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_C6_Sum[ p->ID ].setDB( &(*p) );
          //N_C6_SUM++;
          if ( N_C6_SUM <= p->ID )
            N_C6_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "FRONTBARREL" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_FRONTBARREL_SUM ){
        //std::cout << " FBAR Sum ID=" << p->ID << std::endl;
        if ( m_FrontBarrel_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for FrontBarrel_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_FrontBarrel_Sum[ p->ID ].setDB( &(*p) );
          //N_FRONTBARREL_SUM++;
          if ( N_FRONTBARREL_SUM <= p->ID )
            N_FRONTBARREL_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CC01" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CC01_SUM ){
        //std::cout << " CC01 Sum ID=" << p->ID << std::endl;
        if ( m_CC01_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CC01_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CC01_Sum[ p->ID ].setDB( &(*p) );
          //N_CC01_SUM++;
          if ( N_CC01_SUM <= p->ID )
            N_CC01_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CC02" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CC02_SUM ){
        //std::cout << " CC02 Sum ID=" << p->ID << std::endl;
        if ( m_CC02_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CC02_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CC02_Sum[ p->ID ].setDB( &(*p) );
          //N_CC02_SUM++;
          if ( N_CC02_SUM <= p->ID )
            N_CC02_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "MAINBARREL" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_MAINBARREL_SUM ){
        //std::cout << " MBAR Sum ID=" << p->ID << std::endl;
        if ( m_MainBarrel_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for MainBarrel_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_MainBarrel_Sum[ p->ID ].setDB( &(*p) );
          //N_MAINBARREL_SUM++;
          if ( N_MAINBARREL_SUM <= p->ID )
            N_MAINBARREL_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "COSMIC" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_COSMICCOUNTER_SUM ){
        //std::cout << " MBAR Sum ID=" << p->ID << std::endl;
        if ( m_CosmicCounter_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for CosmicCounter_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_CosmicCounter_Sum[ p->ID ].setDB( &(*p) );
          //N_COSMICCOUNTER_SUM++;
          if ( N_COSMICCOUNTER_SUM <= p->ID )
            N_COSMICCOUNTER_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "XEMON" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_XEMONITOR_SUM ){
        //std::cout << " XE Sum ID=" << p->ID << std::endl;
        if ( m_XeMonitor_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for XeMonitor_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_XeMonitor_Sum[ p->ID ].setDB( &(*p) );
          //N_XEMONITOR_SUM++;
          if ( N_XEMONITOR_SUM <= p->ID )
            N_XEMONITOR_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "HODOSCOPE" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_HODOSCOPE_SUM ){
        //std::cout << " HODOSCOPE Sum ID=" << p->ID << std::endl;
        if ( m_Hodoscope_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for Hodoscope_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_Hodoscope[ p->ID ].setDB( &(*p) );
          //N_HODOSCOPES_SUM++;
          if ( N_HODOSCOPE_SUM <= p->ID )
            N_HODOSCOPE_SUM = p->ID + 1;
        }
      }
    }
    else if ( p->name == "CVTRG" && p->type == "SUM" ){
      if ( 0 <= p->ID && p->ID < N_MAX_CHARGEDTRIGGER_SUM ){
        //std::cout << " CVTRG_SUM ID=" << p->ID << std::endl;
        if ( m_ChargedTrigger_Sum[ p->ID ].pDB() ){
          std::cout << "E391Detector::getDB :"
                    << " multiple definition for ChargedTrigger_Sum#" << p->ID
                    << " : latter data discarded."
                    << std::endl;
        }
        else{
          m_ChargedTrigger_Sum[ p->ID ].setDB( &(*p) );
          //N_HODOSCOPE++;
          if ( N_CHARGEDTRIGGER_SUM <= p->ID )
            N_CHARGEDTRIGGER_SUM = p->ID + 1;
        }
      }
    }

  }
  // dump
  if ( DEBUG )
    m_detparam->dumpDB();

  if ( DEBUG ){
    std::cout << "E391Detector::init_param()" << std::endl
              << "N_C6  =" << N_C6 << std::endl
              << "N_FB  =" << N_FRONTBARREL << std::endl
              << "N_CC01=" << N_CC01 << std::endl
              << "N_CC02=" << N_CC02 << std::endl
              << "N_MB  =" << N_MAINBARREL << std::endl
              << "N_CV  =" << N_CHARGEDVETO << std::endl
              << "N_CSI =" << N_CSI << std::endl
              << "N_Sandwich=" << N_SANDWICH << std::endl
              << "N_CC03=" << N_CC03 << std::endl
              << "N_CC04=" << N_CC04 << std::endl
              << "N_CC05=" << N_CC05 << std::endl
              << "N_BeamCatcher=" << N_BEAMCATCHER << std::endl
              << "N_CosmicCounter=" << N_COSMICCOUNTER << std::endl
              << "N_XeMonitor=" << N_XEMONITOR << std::endl
              << "N_Hodoscope=" << N_HODOSCOPE << std::endl
              << "N_ChargedTrigger=" << N_CHARGEDTRIGGER << std::endl;
    
  }

  return( 0 );
}


//
void
E391Detector::setEvent( EVENT_HEADER* pheader, void* pevent )
{
  m_pheader = pheader;
  m_pevent  = pevent;
}


//
int
E391Detector::unpack( int nth )
{
  if ( m_pheader && m_pevent ){
    return( unpack( m_pheader, m_pevent, nth ) );
  }
  return( -1 );
}


//
int
E391Detector::unpack( EVENT_HEADER* pheader, void* pevent, int nth )
{
  if ( DEBUG )
    std::cout << "E391Detector::unpack()" << std::endl;
  //
  delete_param();

  //
  m_pheader = pheader;
  m_pevent  = pevent;

  // banks
  DWORD*  pdataTDC0=0;
  int  ndataTDC0 = bk_locate( m_pevent, "TDC0", &pdataTDC0 );
  DWORD*  pdataTDC1=0;
  int  ndataTDC1 = bk_locate( m_pevent, "TDC1", &pdataTDC1 );

  DWORD*  pdataADC0=0;
  int  ndataADC0 = bk_locate( m_pevent, "ADC0", &pdataADC0 );
  DWORD*  pdataADC1=0;
  int  ndataADC1 = bk_locate( m_pevent, "ADC1", &pdataADC1 );

  DWORD*  pdataSMP0=0;
  int  ndataSMP0 = bk_locate( m_pevent, "SMP0", &pdataSMP0 );
  DWORD*  pdataSMP1=0;
  int  ndataSMP1 = bk_locate( m_pevent, "SMP1", &pdataSMP1 );

  DWORD*  pdataNGF0=0;
  int  ndataNGF0 = bk_locate( m_pevent, "NGF0", &pdataNGF0 );
  DWORD*  pdataNGF1=0;
  int  ndataNGF1 = bk_locate( m_pevent, "NGF1", &pdataNGF1 );

  DWORD*  pdataTRG0=0;
  int  ndataTRG0 = bk_locate( m_pevent, "TRG0", &pdataTRG0 );
  DWORD*  pdataTRG1=0;
  int  ndataTRG1 = bk_locate( m_pevent, "TRG1", &pdataTRG1 );

  DWORD*  pdataFTD0=0;
  int  ndataFTD0 = bk_locate( m_pevent, "FTD0", &pdataFTD0 );
  DWORD*  pdataFTD1=0;
  int  ndataFTD1 = bk_locate( m_pevent, "FTD1", &pdataFTD1 );

  DWORD*  pdataSCL0=0;
  int  ndataSCL0 = bk_locate( m_pevent, "SCL0", &pdataSCL0 );

  int  ret = 0;

  //
  m_tdcCrate[0] = new TdcEvent( 0, m_pheader, pdataTDC0, ndataTDC0, nth );
  if ( m_tdcCrate[0]->serial() == nth ){
    m_tdcCrate[0]->smp( pdataSMP0,ndataSMP0 ); // SMP
  }
  else{
    ret = -1;
  }

  //
  m_tdcCrate[1] = new TdcEvent( 1, m_pheader, pdataTDC1, ndataTDC1, nth );
  if ( m_tdcCrate[1]->serial() == nth ){
    m_tdcCrate[1]->smp( pdataSMP1,ndataSMP1 ); // SMP
  }
  else{
    ret = -1;
  }

  //
  m_adcCrate[0] = new AdcEvent( 0, m_pheader, pdataADC0, ndataADC0, nth );
  if ( m_adcCrate[0]->serial() == nth ){
    m_adcCrate[0]->ngf( pdataNGF0,ndataNGF0 );
    m_adcCrate[0]->trg( pdataTRG0,ndataTRG0 );
  }
  else{
    ret = -1;
  }

  //
  m_adcCrate[1] = new AdcEvent( 1, m_pheader, pdataADC1, ndataADC1, nth );
  if ( m_adcCrate[1]->serial() == nth ){
    m_adcCrate[1]->ngf( pdataNGF1,ndataNGF1 );
    m_adcCrate[1]->trg( pdataTRG1,ndataTRG1 );
  }
  else{
    ret = -1;
  }

  //
  m_mtdcCrate[0] = new MTdcEvent( 0, m_pheader, pdataFTD0, ndataFTD0, nth );
  if ( m_mtdcCrate[0]->serial() == nth ){
    ; // OK
  }
  else{
    if ( ndataFTD0 > 0 )
      ret = -1;
  }
  m_mtdcCrate[1] = new MTdcEvent( 1, m_pheader, pdataFTD1, ndataFTD1, nth );
  if ( m_mtdcCrate[1]->serial() == nth ){
    ; // OK
  }
  else{
    if ( ndataFTD1 > 0 )
      ret = -1;
  }

  //
  return( ret );
}


//
int
E391Detector::spillNumber() const
{
  const int*  pev;
  int  n = bank( "EVNT",&pev );
  if ( pev && n >= 2 )
    return( pev[2] );
  return( m_pheader->serial_number );
}


//
int
E391Detector::eventNumber() const
{
  const int*  pev;
  int  n = bank( "EVNT",&pev );
  if ( pev && n >= 3 )
    return( pev[3] );
  return( -1 );
}


int
E391Detector::eventTime() const
{
  if ( m_pheader )
    return( m_pheader->time_stamp );
  return( 0 );
}


//
int
E391Detector::triggerBit( int nth ) const
{
  const int*  pev;
  int  n = bank( "TRG1",&pev );
  if ( pev && n >= nth )
    return( pev[nth-1] );
  return( 0 );
}


//
int
E391Detector::bank( const std::string& bankname, void* pdata ) const
{
  if ( DEBUG )
    std::cout << "E391Detector::bank()" << std::endl;
  
  if ( m_pevent )
    return( bk_locate( m_pevent, (char*)bankname.c_str(),pdata ) );
  return( 0 );
}


//
bool
E391Detector::exist( const std::string& bankname ) const
{
  if ( DEBUG )
    std::cout << "E391Detector::exist()" << std::endl;

  if ( m_pevent ){ 
    DWORD*  pdata;
    int  n = bk_locate( m_pevent, (char*)bankname.c_str(), &pdata );
    if ( n ){
      return( true );
    }
  }
  return( false );
}

