// -*- C++ -*-
//
// MidasBank.h
// Midas Bank Manager class
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Jul 10 18:40:57 JST 2001
//
// $Id: MidasBank.h,v 1.2 2002/08/29 13:25:34 yamaga Exp $
//
// Revision History
//
// $Log: MidasBank.h,v $
// Revision 1.2  2002/08/29 13:25:34  yamaga
// Many functions added and modified...
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#ifndef MIDASBANK_H_INCLUDED
#define MIDASBANK_H_INCLUDED

#include <list>
#include <string>
#include "midas.h"


//////
class MidasBank {
public:
  MidasBank( int bufsize = EVENT_BUFFER_SIZE );
  MidasBank( const MidasBank& );
  ~MidasBank();
  //
  MidasBank&  operator=( const MidasBank& );
  //
  inline       void*  eventBuffer()       { return (void*)(eventHeader()+1); }
  inline const void*  eventBuffer() const { return (void*)(eventHeader()+1); }
  inline       EVENT_HEADER*  eventHeader();
  inline const EVENT_HEADER*  eventHeader() const;
  inline int          bufsize() const { return m_bufsize; }
  bool                realloc( int size );
  //
  void              init( int event_id, int trigger_mask, int serial_number );
  void              initFilter( const EVENT_HEADER* pheader );
  std::list<BANK32> getBankDefs();
  int               getBank( const std::string& name, void* pdata );
  void*             createBank( const std::string& name, 
                                int TID_type, int n_elements=0 );
  int               closeBank( void* pdata );
  int               addBank( const std::string& name, int TID_type, 
                             int n_elements, void* pdata );
  int               copyBank( const MidasBank* bank, const std::string& name );
  int               copyAllBank( const MidasBank* bank );
  
  void              showBankDefs();

private:
  int     m_bufsize;
  char*   m_unalignedbuf;
  char*   m_unalignedbuf_w;
  //
};

// inline
inline EVENT_HEADER*
MidasBank::eventHeader()
{
  return (EVENT_HEADER*)ALIGN((PTYPE)m_unalignedbuf);
}

inline const EVENT_HEADER*
MidasBank::eventHeader() const
{
  return (EVENT_HEADER*)ALIGN((PTYPE)m_unalignedbuf);
}

#endif //MIDASBANK_H_INCLUDED
