// -*- C++ -*-
//
// TMNtuple.h
// NTUPLE accessing classes
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Sep 11 17:17:42 JST 2001
//
// $Id: TMNtuple.h,v 1.2 2002/11/20 06:07:29 yamaga Exp $
//
// Revision History
//
// $Log: TMNtuple.h,v $
// Revision 1.2  2002/11/20 06:07:29  yamaga
// addVars() added to use large # of variables in CWN.
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#ifndef TMNTUPLE_H_INCLUDED
#define TMNTUPLE_H_INCLUDED

//
#include <string>


//
class TupleManager;


//////
class TMNtuple {
public:
  TMNtuple( const TupleManager* );
  ~TMNtuple();
  //
  // CWN
  void  bookCWN( int hid, const std::string& title,
                 const std::string& block, 
                 void* vars, const std::string& cname );
  // CWN, read only
  void  bookCWN( int hid, const std::string& block, void* vars );

  // RWN by vars
  void  bookRWN( int hid, const std::string& title,
                 void* vars, const std::string& varname );
  // RWN by name
  void  bookRWN( int hid, const std::string& title,
                 const std::string& varname );
  // RWN by vars, read only
  void  bookRWN( int hid, void* vars );

  // Add variables to the CWN
  void  addVars( const std::string& block,
                 void* vars, const std::string& varname );

  // Fill RWN members by name
  void  fill( const std::string& vname, float val );

  // dump data buffer to the output file
  int   dump();

  // RWN by name, clear the data
  void  clear();
  
  // read data block into &vars
  int   read( int eventNo );

  // hnoent()
  int   totalEntries();

private:
  //
  bool          m_bufinit;
  int           m_type; // TMN_CWN or TMN_RWN, see TupleManager.h
  std::string   m_title;
  std::string   m_block;
  std::string   m_cname;
  int           m_hid;
  int           m_nvars;
  float*        m_vars;
  int*          m_varindex; // RWN by internal vars
  std::string*  m_varname;  // RWN by internal vars
  const TupleManager* m_manager;
  
};


#endif // TMNTUPLE_H_INCLUDED

