// -*- C++ -*-
//
// TupleManager.h
// HBOOK/NTUPLE manager class
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Sep 11 17:17:42 JST 2001
//
// $Id: TupleManager.h,v 1.2 2002/10/29 09:47:09 yamaga Exp $
//
// Revision History
//
// $Log: TupleManager.h,v $
// Revision 1.2  2002/10/29 09:47:09  yamaga
// PAW global memory available by initMap().
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#ifndef TUPLEMANAGER_H_INCLUDED
#define TUPLEMANAGER_H_INCLUDED

//
#include <string>
#include <list>


//
#define  TM_Write    1
#define  TM_Read    -1
#define  TM_Shared   2
#define  TM_CWN      1
#define  TM_RWN      2
#define  TM_RWNNAME  3


//
#include "TupleManager/TMNtuple.h"
#include "TupleManager/TMHistogram.h"


//////
class TupleManager {
public:
  TupleManager();
  ~TupleManager();
  //
  const std::string&  fname()  const { return m_fname;  }
  const std::string&  dir()    const { return m_dir;    }
  const std::string&  cdir()   const { return m_cdir;   }
  int                 lun()    const { return m_lun;    }
  int                 status() const { return m_status; }
  static bool         isShared()     { return s_shared; }
  //
  int  ropen( const std::string& fname, const std::string& pawdir,int lun );
  int  wopen( const std::string& fname, const std::string& pawdir,int lun );
  int  close();
  int  write( int hid = 0 );
  //
  // ntuple for output; title_name,block_name,&vars and varstring required
  TMNtuple*     ntuple( int hid, const std::string& title,
                        const std::string& block, 
                        void* vars, const std::string& cname );
  // ntuple for intput; block_name and &vars only required
  TMNtuple*     ntuple( int hid, const std::string& block, void* vars );
  // RWN
  TMNtuple*     ntupleR( int hid, const std::string& title,
                         void* vars, const std::string& cname );
  TMNtuple*     ntupleR( int hid, const std::string& title,
                         const std::string& cname );
  // RWN for input
  TMNtuple*     ntupleR( int hid, void* vars );
  // 1-D hist
  TMHistogram*  histogram( int hid, const std::string& title,
                           int xbin,double xmin, double xmax );
  // 2-D hist
  TMHistogram*  histogram( int hid, const std::string& title,
                           int xbin,double xmin, double xmax,
                           int ybin,double ymin, double ymax );
  //
  static void  init( int hlimit );
  static void  initMap( int hlimit, const std::string& mapname );

private:
  static bool  s_shared;
  std::string  m_fname;
  std::string  m_dir;
  std::string  m_cdir;
  int          m_lun;
  int          m_status;
  std::list<TMNtuple*>     m_ntlist;
  std::list<TMHistogram*>  m_hilist;
  //
};


#endif // TUPLEMANAGER_H_INCLUDED

