// -*- C++ -*-
//
// Detector.h
// Template header file
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Aug  6 09:22:10 JST 2002
//
// $Id: Detector.h,v 1.4 2002/12/04 10:03:20 yamaga Exp $
//
// Revision History
//
// $Log: Detector.h,v $
// Revision 1.4  2002/12/04 10:03:20  yamaga
// Updates for MultihitTDC.
//
// Revision 1.3  2002/09/20 14:35:36  yamaga
// pDB() function added.
//
// Revision 1.2  2002/08/29 14:13:03  yamaga
// Function name 'charge' changed to 'energy'
//
// Revision 1.1.1.1  2002/08/13 12:48:13  kensh
//
//
// Start version.
//
//

#ifndef DETECTOR_H_INCLUDED
#define DETECTOR_H_INCLUDED

#include "detparam/DetParam.h"
#include "unpack/MTdcModule.h"

//
#define ERRVAL -999


//
class Detector{
public:
  //
  Detector();
  ~Detector();
  //
  int       adc() const;
  int       tdc() const;
  double    energy() const;
  double    time() const;
  const std::vector<LRS1877SDATA>&  mtdc() const;
  const std::vector<double>&        mtime() const;
  
  //
  int   adcCrate() const { return (m_DB ? m_DB->adcCrate   : -1); }
  int   adcSlot()  const { return (m_DB ? m_DB->adcSlot    : -1); }
  int   adcCh()    const { return (m_DB ? m_DB->adcChannel : -1); }
  int   tdcCrate() const { return (m_DB ? m_DB->tdcCrate   : -1); }
  int   tdcSlot()  const { return (m_DB ? m_DB->tdcSlot    : -1); }
  int   tdcCh()    const { return (m_DB ? m_DB->tdcChannel : -1); }
  int   mtdcCrate() const { return (m_DB ? m_DB->mtdcCrate   : -1); }
  int   mtdcSlot()  const { return (m_DB ? m_DB->mtdcSlot    : -1); }
  int   mtdcCh()    const { return (m_DB ? m_DB->mtdcChannel : -1); }
  //
  void    setDB( const E391_DETE_DB* dbptr ) { m_DB = dbptr; }
  const E391_DETE_DB*  pDB() const { return m_DB; }
  //

private:
  // data
  const E391_DETE_DB*  m_DB;
  // method
};

//
#endif // DETECTOR_H_INCLUDED

