// -*- C++ -*-
//
// DetParam.h
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Aug  6 09:22:10 JST 2002
//
// $Id: DetParam.h,v 1.6 2002/12/04 10:03:46 yamaga Exp $
//
// Revision History
//
// $Log: DetParam.h,v $
// Revision 1.6  2002/12/04 10:03:46  yamaga
// Updates for MultihitTDC.
//
// Revision 1.5  2002/12/01 14:27:42  yamaga
// Updated for multihit TDC.
//
// Revision 1.4  2002/11/20 06:49:30  yamaga
// Added pedestalDB
//
// Revision 1.3  2002/09/27 12:15:07  yamaga
// Conform to updated DB format.
//
// Revision 1.2  2002/08/29 14:17:42  yamaga
// Conform to DB format.
//
// Revision 1.1.1.1  2002/08/13 12:48:13  kensh
//
//
// Start version.
//
//

#ifndef DETPARAM_H_INCLUDED
#define DETPARAM_H_INCLUDED

#include <iostream>
#include <list>
#include <string>

//
#define  MAX_NADCCRATE   2
#define  MAX_NADC       23
#define  MAX_NADCCH     96
#define  MAX_NTDCCRATE   2
#define  MAX_NTDC       23
#define  MAX_NTDCCH     64
#define  MAX_NMTDCCRATE  2
#define  MAX_NMTDC      23
#define  MAX_NMTDCCH    96


// # det type subID ID ADCpa ADCsa TDCma TDCsa x y z HVcr HVsl HVch gain pedL pedH Tresol
typedef struct{
  std::string  name;
  std::string  type;
  int          clusterID;
  int          ID;
  int          adcCrate;
  int          adcSlot;
  int          adcChannel;
  int          tdcCrate;
  int          tdcSlot;
  int          tdcChannel;
  double       x;
  double       y;
  double       z;
  int          hvCrate;
  int          hvSlot;
  int          hvChannel;
  double       hvVset;
  double       hvIset;
  double       gainL;
  double       pedestalL;
  double       pedWidthL;
  double       gainH;
  double       pedestalH;
  double       pedWidthH;
  double       time0;
  double       timeResol;
  int          mtdcCrate;
  int          mtdcSlot;
  int          mtdcChannel;
  double       mtime0;
  double       mtimeResol;
} E391_DETE_DB;


typedef struct{
  int          adcCrate;
  int          adcSlot;
  int          adcChannel;
  double       pedestalL;
  double       pedWidthL;
  double       pedestalH;
  double       pedWidthH;
} ADC_PEDESTAL_DB;


//
class DetParam{
public:
  // singleton
  static DetParam&  instance();
  // extractor
  const std::list<E391_DETE_DB>&  paramList() const { return m_paramList; }
  const std::list<ADC_PEDESTAL_DB>&  pedestalList() const { return m_pedestalList; }
  // reverse index
  const E391_DETE_DB*  db_adc( int crate, int slot, int ch ) const;
  const E391_DETE_DB*  db_tdc( int crate, int slot, int ch ) const;
  const E391_DETE_DB*  db_mtdc( int crate, int slot, int ch ) const;
  const ADC_PEDESTAL_DB*  db_adcped( int crate, int slot, int ch ) const;
  // DB
  bool  getDB( int runno );
  bool  getPedDB( int runno );
  void  dumpDB() const;

protected:
  DetParam();
  ~DetParam();

private:
  // singleton
  static DetParam*  s_detparam;
  // data
  std::list<E391_DETE_DB>  m_paramList;
  std::list<ADC_PEDESTAL_DB>  m_pedestalList;
  E391_DETE_DB*  m_db_adc[MAX_NADCCRATE][MAX_NADC][MAX_NADCCH]; //reverse index
  E391_DETE_DB*  m_db_tdc[MAX_NTDCCRATE][MAX_NTDC][MAX_NTDCCH]; //reverse index
  E391_DETE_DB*  m_db_mtdc[MAX_NTDCCRATE][MAX_NTDC][MAX_NTDCCH];//reverse index
  ADC_PEDESTAL_DB*  m_db_adcped[MAX_NADCCRATE][MAX_NADC][MAX_NADCCH]; //reverse index
  // method
  void    makeReverseIndex();
  void    makeReverseIndexPed();

};

//
#endif // DETPARAM_H_INCLUDED

