/* -*- C -*-

   gufio.h

   Author:  Mitsuhiro YAMAGA
   Created: Wed Dec 11 22:57:06 JST 2002

   $Id: gufio.h,v 1.1 2002/12/11 14:11:07 yamaga Exp $

   Revision History

   $Log: gufio.h,v $
   Revision 1.1  2002/12/11 14:11:07  yamaga
   Imported package.


*/
#ifndef __H_GUFILE
#define __H_GUFILE

#include <sys/stat.h>
#include <unistd.h>

#ifdef __cplusplus
extern"C"{
#endif
/*
         Grand Unified File IO (GUFIO) utility routines
         GUFIO switches multiple protocols by looking at the protocol field,
         which start with / and end with : placed at the head of
         the file name.   Currently supported protocols are
         FS(file system) and HPSS(high performance storage system).

         /hpss:<hpss name>  -> invoke hpss calls for the file <hpss name>
         /:                 -> expanded to /.../cc.kek.jp/fs
                                              (hard wired for the momtne)
                               and use the system standard open call.
         anything else      -> use the system standard open call.

*/	 

static char rcsid_gufio_h[] = "@(#) $Id: gufio.h,v 1.1 2002/12/11 14:11:07 yamaga Exp $";


typedef int gufio_t;

#define TYPICALFILESIZE 100000000

#define GUFILE_NONE 0
#define GUFILE_HPSS 1
#define GUFILE_FS   2


struct gufio_tbl {
  struct gufio_tbl *next;
  int fd;
  gufio_t ftype;
  int hfd;
  int position;
};

extern struct gufio_tbl *gutblput(int,gufio_t,int);
extern int gutblrm(int);
extern struct gufio_tbl *gutblget(int);
extern struct gufio_tbl *gutblnext(struct gufio_tbl*);
extern struct gufio_tbl *gutblfirst(void);
extern struct gufio_tbl *gutblinit(void);

extern int guexpandpath(const char*,char*);
extern int gubufsiz(int,int);
extern int gufcntl(int,int,int);
extern int gucreat(const char*,mode_t);
extern int guopen(const char*,int,mode_t);
extern int guclose(int);
extern int guread(int,void*,size_t);
extern int guwrite(int,void*,size_t);
extern off_t gulseek(int,off_t,int);
extern int gustat(const char*,struct stat*);
extern void guexit(void);
extern int gureopenall(void);

#ifdef __cplusplus
}
#endif

#endif
