/*                    Grand Unified CLOSE
         Switch multiple protocols by looking at the protocol field
         which start with / and end with : placed at the head of
         the file name.   Currently supported protocols are
         FS(file system) and HPSS(high performance storage system).

         /hpss:<hpss name>  -> invoke hpss calls for the file <hpss name>
         /:                 -> expanded to /.../cc.kek.jp/fs
                                              (hard wired for the momtne)
                               and use the system standard close call.
         anything else      -> use the system standard close call.

	 
	 This code based on the hpsscat.c provided by IBM Japan (2001/01/23).
         M. Aoki modified the base code (2001/01/23).
*/	 

static char rcsid_guclose[] = "@(#) $Id: guclose.c,v 1.1 2002/12/11 14:12:33 yamaga Exp $";


#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include "hpss_api.h"
#include "hpss_types.h"
#include "u_signed64.h"
#include "gufio/hpss_bufapi.h"
#include "gufio/gufio.h"


int guclose(int fd)
{
  struct gufio_tbl *thetbl;
  int status;
  int nlinks;
  struct gufio_tbl *ptr;
  int thehfd;

  thetbl = gutblget(fd);
  if ( thetbl->ftype == GUFILE_HPSS ) {
    /* Check if there is no other gutbl pointing to the hfd */
    thehfd = thetbl->hfd;
    nlinks = 0;
    if ( ( ptr = gutblfirst() ) != NULL ) {
      do {
	if ( ( ptr->ftype == GUFILE_HPSS ) && ptr->hfd == thehfd ) {
	  nlinks++;
	}
      } while ( (ptr = ptr->next) != NULL );
    }

    if ( nlinks == 1 ) {
      status = hpss_Bufclose(thetbl->hfd);
      if ( status < 0 ) {
	/*return(-1);*/  /* errno is supposed to be set by hpss_Bufclose */
        gureopenall();
        status = hpss_Bufclose(thetbl->hfd);
      }
    }

    /* close a dummy file descripter */
    status = close(fd);
    if ( status < 0 ) {
      return(-1);  /* errno is supposed to be set by close */
    }
  } else {
    status = close(thetbl->hfd);
    if ( status < 0 ) {
      return(-1);  /* errno is supposed to be set by close */
    }
  }

  status = gutblrm(fd);
  return(status);
}
