/*                    Grand Unified CREAT
         Switch multiple protocols by looking at the protocol field
         which start with / and end with : placed at the head of
         the file name.   Currently supported protocols are
         FS(file system) and HPSS(high performance storage system).

         /hpss:<hpss name>  -> invoke hpss calls for the file <hpss name>
         /:                 -> expanded to /.../cc.kek.jp/fs
                                              (hard wired for the momtne)
                               and use the system standard creat call.
         anything else      -> use the system standard creat call.

	 
	 This code based on the hpsscat.c provided by IBM Japan (2001/01/23).
         M. Aoki modified the base code (2001/01/23).
*/	 

static char rcsid_gucreat[] = "@(#) $Id: gucreat.c,v 1.1 2002/12/11 14:12:33 yamaga Exp $";


#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <stdlib.h>
#include "hpss_api.h"
#include "hpss_types.h"
#include "u_signed64.h"
#include "gufio/hpss_bufapi.h"
#include "gufio/gufio.h"


int gucreat(const char *filename, mode_t mode)
{
  char *body;
  int protocol;
  int fd, hfd;
  int tmp_errno, status;

  body = (char *)malloc((strlen(filename)+100)*sizeof(char));
  if ( ( protocol = guexpandpath(filename,body) ) < 0 ) {
    errno = EACCES; /* Wrong protocol */
    return(-1);
  }

  if ( protocol == GUFILE_HPSS ) {
    /* create the source file in HPSS */
    hfd = hpss_Bufcreate(body, O_RDWR, mode, TYPICALFILESIZE);
    if(hfd < 0)
      {
	return(-1);  /* errno is supposed to be set by hpss_Create */
      }

    /* open a dummy file */
    /* It looks like HPSS uses its own numbering in the file descriptor.
       Thus, in order to handle the files seemlessly between the standard
       file system and HPSS, it is necessary to lock one file descriptor
       of the stardard file system per one HPSS open. */
    fd = open("/dev/null", O_RDWR, 600);
    if(fd < 0)
      {
	tmp_errno = errno;             /* save errno */
	status = hpss_Bufclose(hfd);
	errno = tmp_errno;             /* restore errno */
	return(-1);  /* errno is supposed to be set by open */
      }

    if ( gutblput(fd,GUFILE_HPSS,hfd) == NULL ) {
      errno = EBADF; /* Bad file descriptor */
      return(-1);
    }

    /* At exit stage, all HPSS opened files should be properly closed.
       Otherwise it may screw-up the file contents.
       For example, hpss_opened-but-not-closed file usually have
       an excesive bytes. */
    if ( atexit( (*guexit) ) ) {
      return(-1);
    }

  } else  if ( protocol == GUFILE_FS ) {
    /* create the source file */
    fd = creat(body, mode);
    if(fd < 0)
      {
	return(-1);  /* errno is supposed to be set by creat */
      }
    if ( gutblput(fd,GUFILE_FS,fd) == NULL ) {
      errno = EBADF; /* Bad file descriptor */
      return(-1);
    }

  } else {
    errno = EACCES; /* Wrong protocol */
    return(-1);
  }

  return(fd);
}
