// -*- C++ -*-
//
// MTdcModule.h
//
// Sun Dec  1 15:06:54 JST 2002
// Mitsuhiro YAMAGA
//
// $Id: MTdcModule.h,v 1.2 2002/12/04 10:06:45 yamaga Exp $
//
// $Log: MTdcModule.h,v $
// Revision 1.2  2002/12/04 10:06:45  yamaga
// Updates for MultihitTDC.
//
// Revision 1.1  2002/12/01 14:34:51  yamaga
// Initial version for multihit TDC.
//
//
//

#ifndef MTDCMODULE_H_INCLDUED
#define MTDCMODULE_H_INCLDUED

#include <iostream>
#include <vector>
#include <list>

#define  MAX_MTDC_CH  96

#define  LEADIN_EDGE    0
#define  TRAILING_EDGE  1
#define  RISING_EDGE    0
#define  FALLING_EDGE   1

typedef struct{
  int  crate;
  int  pa;
  int  parity;
  int  hitct;
  int  ch;
  int  edge;
  int  time;
} LRS1877SDATA;


//
class MTdcModule {
public:
  // constructor
  MTdcModule( int crate=-1, int pa=-1 );
  ~MTdcModule();
  // extractor
  inline int           crate()        const { return m_crate;        }
  inline int           pa()           const { return m_pa;           }
  inline unsigned int  headerWord()   const { return m_headerWord;   }
  inline int           parity()       const { return m_parity;       }
  inline int           bufferNumber() const { return m_bufferNumber; }
  inline int           wordCount()    const { return m_wordCount;    }
  inline const std::vector<LRS1877SDATA>&  data( int ch ) const;
  inline const std::list<unsigned int>&  hitList() const { return m_hitList; }
  inline       std::list<unsigned int>&  hitList()       { return m_hitList; }
  inline int   nhitChannel()  const ;
  inline int   nhit()  const ;
  inline unsigned long long  hitPatternL() const;
  inline unsigned long long  hitPatternH() const;
  void         show() const;
  // modifier
  inline void  setCrate( int crate ){ m_crate = crate; }
  inline void  setPa( int pa ) { m_pa = pa; }
  inline void  setHeaderWord( unsigned data );
  inline void  setData( unsigned int data );
  // friend
  friend std::ostream&  operator<<( std::ostream& out, const MTdcModule& m );

private:
  // data
  int  m_crate;
  int  m_pa;
  int  m_parity;
  int  m_bufferNumber;
  int  m_wordCount;
  unsigned int  m_headerWord;
  std::vector<LRS1877SDATA>  m_ch[MAX_MTDC_CH];
  std::list<unsigned int>    m_hitList;

};


// inline
//
inline int
MTdcModule::nhitChannel() const
{
  int  ct = 0;
  for( int i=0;i<MAX_MTDC_CH;i++ ){
    if ( m_ch[i].size() > 0 )
      ct++;
  }
  return( ct );
}

inline int
MTdcModule::nhit() const
{
  //int  ct = 0;
  //for( int i=0;i<MAX_MTDC_CH;i++ ){
  //  ct += m_ch[i].size();
  //}
  //return( ct );
  return( m_hitList.size() );
}

//
inline unsigned long long
MTdcModule::hitPatternL() const
{
  unsigned long long  pat = 0;
  for( int i=47;i>=0;i-- ){
    pat <<= 1;
    if ( m_ch[i].size() > 0 )
      pat |= 1;
  }
  return( pat );
}

//
inline unsigned long long
MTdcModule::hitPatternH() const
{
  unsigned long long  pat = 0;
  for( int i=95;i>=48;i-- ){
    pat <<= 1;
    if ( m_ch[i].size() > 0 )
      pat |= 1;
  }
  return( pat );
}

//
inline const std::vector<LRS1877SDATA>&
MTdcModule::data( int ch ) const
{
  if ( 0 <= ch && ch < MAX_MTDC_CH )
    return( m_ch[ch] );
  std::cout << "MTdcModule::data : ch# out of range." << std::endl;
  return( m_ch[0] );
}


//
inline void
MTdcModule::setHeaderWord( unsigned int data )
{
  m_headerWord   = data;
  m_pa           = (data >> 27) & 0x1f;
  m_parity       = (data >> 15) & 0x01;
  m_bufferNumber = (data >> 11) & 0x07;
  m_wordCount    =  data & 0x07ff;
}

//
inline void
MTdcModule::setData( unsigned int data )
{
  int  pa     = (data >> 27) & 0x1f;
  int  parity = (data >> 26) & 0x01;
  int  hitct  = (data >> 24) & 0x03;
  int  ch     = (data >> 17) & 0x7f;
  int  edge   = (data >> 16) & 0x01;
  int  time   =  data & 0xffff;
  if ( 0 <= ch && ch <= MAX_MTDC_CH ){
    LRS1877SDATA  dt = {
      m_crate,
      m_pa,
      parity,
      hitct,
      ch,
      edge,
      time
    };
    m_ch[ch].push_back( dt );
  }
}


#endif // TDCMODULE_H_INCLDUED
